import numpy as np
import pandas as pd
import os
import csv
import warnings
import sys


def DataFrame_from_full_data(root_dir):
    """Creates Pandas Dataframe from the three numpy arrays created 
    with the function `load_full_data` from file `load_data_local_numpy`.
    This DataFrame will be the actual data to work with.
    
    Arguments:
        root_dir {str}:
            Folder which contains the three numpy arrays.
    
    Returns:
        data {pandas.DataFrame}:
            DataFrame containing the data with the trajectories as rows
            and the features as columns.
    """

    # Load Numpy files
    data = np.load(os.path.join(root_dir, 'data.npy'))
    hyperparameters = np.load(os.path.join(root_dir, 'hyperparameters.npy'))
    features = np.load(os.path.join(root_dir, 'features.npy'))

    # if necessary, correct numpy encoding from bytes to strings
    if type(features[0]) == np.bytes_:
        features = [feat.decode() for feat in features]

    # create DataFrame
    data = pd.DataFrame(data, columns = features)

    index = np.zeros((2, 0))

    # create Multiindex from hyperparameters
    # if necessary, correct numpy encoding from bytes to strings
    if type(hyperparameters[0, 0]) == np.bytes_:
        for hyperparameter in hyperparameters:
            new_index = np.zeros((2, int(hyperparameter[3]))).astype('str')
            new_index[1, :] = np.arange(int(hyperparameter[3]))

            new_index[0, :] = int(hyperparameter[3]) * \
                                    [hyperparameter[0].decode() + ':' + \
                                    hyperparameter[1].decode() + ':' + \
                                    hyperparameter[2].decode()]

            index = np.concatenate([index, new_index], axis = 1)

    else:
        for hyperparameter in hyperparameters:
            new_index = np.zeros((2, int(hyperparameter[3]))).astype('str')
            new_index[1, :] = np.arange(int(hyperparameter[3]))

            new_index[0, :] = int(hyperparameter[3]) * \
                                    [hyperparameter[0] + ':' + \
                                    hyperparameter[1] + ':' + \
                                    hyperparameter[2]]

            index = np.concatenate([index, new_index], axis = 1)

    data.index = pd.MultiIndex.from_arrays(index, 
                                    names = ('hyperparameters', 'snapshot'))
    
    return data
